.flexbox-panel {
    display: flex;
    flex-direction: row;
    flex-wrap: wrap;
    justify-content: center;
    align-items: flex-start;
    align-content: center;
}

.flexbox-item {
    flex: 0 1 auto;
    margin: 1em;
}

.flex-item:nth-child(1) {
    flex-grow: 0;
    flex-shrink: 1;
    flex-basis: auto;
}

.flex-item:nth-child(2) {
    flex-grow: 0;
    flex-shrink: 1;
    flex-basis: auto;
}

.flex-item:nth-child(3) {
    flex-grow: 0;
    flex-shrink: 1;
    flex-basis: auto;
}